from flask import Flask, request, jsonify
import os
import json
from functools import wraps
from datetime import datetime

app = Flask(__name__)
TEXT_FILE = "/srv/http/Requests/requests.txt"
BLOCKED_IPS_FILE = "/srv/http/Requests/blocked_ips.json"
REQUEST_LOG_FILE = "/srv/http/Requests/request_log.json"
CONFIG_FILE = "/srv/http/Requests/titan_config.json"

# Default admin token
ADMIN_TOKEN = "titan_admin_9x4k2p7m5v8n"

# Try to load config if exists
try:
    if os.path.exists(CONFIG_FILE):
        with open(CONFIG_FILE, 'r') as f:
            config = json.load(f)
            if "admin_token" in config:
                ADMIN_TOKEN = config["admin_token"]
except Exception as e:
    print(f"Warning: Using default config. Error loading config: {str(e)}")

def load_request_log():
    if os.path.exists(REQUEST_LOG_FILE):
        try:
            with open(REQUEST_LOG_FILE, 'r') as f:
                return json.load(f)
        except:
            return []
    return []

def save_request_log(logs):
    with open(REQUEST_LOG_FILE, 'w') as f:
        json.dump(logs, f, indent=2)

def log_request(ip, request_text, status="success"):
    logs = load_request_log()
    logs.append({
        "timestamp": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
        "ip": ip,
        "request": request_text,
        "status": status
    })
    # Keep only last 1000 requests to manage file size
    if len(logs) > 1000:
        logs = logs[-1000:]
    save_request_log(logs)
BLOCKED_CONTENT_RESPONSE = "⚠️ Your request was blocked due to inappropriate or spam content. Please use the system responsibly. Continued abuse may result in restrictions."

def load_blocked_ips():
    if os.path.exists(BLOCKED_IPS_FILE):
        try:
            with open(BLOCKED_IPS_FILE, 'r') as f:
                return json.load(f)
        except:
            return []
    return []

def save_blocked_ips(blocked_ips):
    with open(BLOCKED_IPS_FILE, 'w') as f:
        json.dump(blocked_ips, f)

def clean_ip(ip):
    """Remove any http/https prefix from IP address"""
    return ip.replace('https://', '').replace('http://', '')

def check_ip_blocked(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        client_ip = request.remote_addr
        blocked_ips = [clean_ip(ip) for ip in load_blocked_ips()]
        
        if clean_ip(client_ip) in blocked_ips:
            return jsonify({
                "error": BLOCKED_CONTENT_RESPONSE
            }), 403
        return f(*args, **kwargs)
    return decorated_function

@app.route("/api/admin/request-log", methods=["GET"])
def view_request_log():
    # Basic auth check
    auth_token = request.headers.get('X-Admin-Token')
    if not auth_token or auth_token != ADMIN_TOKEN:
        return jsonify({"error": "Unauthorized"}), 401

    # Optional filter by IP
    filter_ip = request.args.get('ip')
    
    logs = load_request_log()
    if filter_ip:
        logs = [log for log in logs if log['ip'] == filter_ip]
    
    return jsonify({"logs": logs}), 200

@app.route("/api/admin/block-ip", methods=["POST"])
def block_ip():
    # Basic auth check - you should implement proper authentication
    auth_token = request.headers.get('X-Admin-Token')
    if not auth_token or auth_token != ADMIN_TOKEN:
        return jsonify({"error": "Unauthorized"}), 401

    data = request.get_json()
    ip = data.get("ip")
    if not ip:
        return jsonify({"error": "No IP provided"}), 400

    # Clean the IP before storing
    clean_ip_addr = clean_ip(ip)
    blocked_ips = load_blocked_ips()
    cleaned_blocked_ips = [clean_ip(x) for x in blocked_ips]
    
    if clean_ip_addr not in cleaned_blocked_ips:
        blocked_ips.append(clean_ip_addr)
        save_blocked_ips(blocked_ips)
        return jsonify({"message": f"IP {clean_ip_addr} blocked successfully"}), 200
    return jsonify({"message": f"IP {clean_ip_addr} was already blocked"}), 200

@app.route("/api/admin/unblock-ip", methods=["POST"])
def unblock_ip():
    # Basic auth check - you should implement proper authentication
    auth_token = request.headers.get('X-Admin-Token')
    if not auth_token or auth_token != ADMIN_TOKEN:
        return jsonify({"error": "Unauthorized"}), 401

    data = request.get_json()
    ip = data.get("ip")
    if not ip:
        return jsonify({"error": "No IP provided"}), 400

    # Clean the IP before checking
    clean_ip_addr = clean_ip(ip)
    blocked_ips = load_blocked_ips()
    blocked_ips = [clean_ip(x) for x in blocked_ips]
    
    if clean_ip_addr in blocked_ips:
        blocked_ips.remove(clean_ip_addr)
        save_blocked_ips(blocked_ips)
        return jsonify({"message": f"IP {clean_ip_addr} unblocked successfully"}), 200
    return jsonify({"message": f"IP {clean_ip_addr} was not blocked"}), 200

@app.route("/api/admin/list-blocked-ips", methods=["GET"])
def list_blocked_ips():
    # Basic auth check - you should implement proper authentication
    auth_token = request.headers.get('X-Admin-Token')
    if not auth_token or auth_token != ADMIN_TOKEN:
        return jsonify({"error": "Unauthorized"}), 401

    blocked_ips = load_blocked_ips()
    return jsonify({"blocked_ips": blocked_ips}), 200

@app.route("/api/requests", methods=["POST"])
@check_ip_blocked
def save_request():
    client_ip = request.remote_addr
    print(f"Headers: {request.headers}")
    print(f"Raw request data: {request.data}")
    try:
        raw_data = request.data.decode("utf-8", errors="ignore")
        print(f"Decoded raw data: {raw_data}")
        data = json.loads(raw_data) if raw_data else {}
        print(f"Parsed JSON: {data}")
        user_request = data.get("request")
        print(f"Extracted request: {user_request}")
        
        # Log the request
        log_request(client_ip, user_request)
        if not user_request:
            print("No 'request' key found or value is empty")
            return jsonify({"error": "No request provided"}), 400
        try:
            with open(TEXT_FILE, "a") as f:
                print(f"Writing '{user_request}' to {TEXT_FILE}")
                f.write(user_request + "\n")
                f.flush()
            print(f"Successfully wrote to {TEXT_FILE}")
            return jsonify({"message": "Request saved successfully"}), 200
        except Exception as e:
            print(f"File write error: {str(e)}")
            return jsonify({"error": f"Failed to save request: {str(e)}"}), 500
    except json.JSONDecodeError as e:
        print(f"JSON decode error: {str(e)}")
        return jsonify({"error": f"Invalid JSON: {str(e)}"}), 400
    except Exception as e:
        print(f"Unexpected error: {str(e)}")
        return jsonify({"error": f"Unexpected error: {str(e)}"}), 400

if __name__ == "__main__":
    # Create necessary directories and files
    for file_path in [TEXT_FILE, BLOCKED_IPS_FILE, REQUEST_LOG_FILE]:
        dir_path = os.path.dirname(file_path)
        if not os.path.exists(dir_path):
            os.makedirs(dir_path)
        if not os.path.exists(file_path):
            print(f"Creating {file_path}")
            if file_path.endswith('.json'):
                with open(file_path, 'w') as f:
                    json.dump([], f)
            else:
                open(file_path, "w").close()
    
    app.run(host="0.0.0.0", port=5001)